<?php
/**
 * Capture Class
 * All necessary functions for capturing an order
 * or retrieving capture information
 *
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.4
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\Lib;
use Bitergo\PaydirektWooCommerce\Lib\Utils;

class Capture {
    const PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX = "https://api.sandbox.paydirekt.de/api/checkout/v1/checkouts";
    const PAYDIREKT_CHECKOUT_ENDPOINT_LIVE = "https://api.paydirekt.de/api/checkout/v1/checkouts";
    const PAYDIREKT_CHECKOUTID_KEY = "checkoutId";
    const PAYDIREKT_CAPTUREID_KEY = "captureId";
    const PAYDIREKT_UTILS_INCLUDE = "utils.php";
    const PAYDIREKT_CODE_KEY = "code";
    const PAYDIREKT_REASON_KEY = "reason";
    const PAYDIREKT_MESSAGE_KEY = "message";
    const PAYDIREKT_DESCRIPTION_KEY = "description";
    const PAYDIREKT_UNPROCESSABLE_ENTITIY = "Unprocessable Entity";
    const STATUS = CAPTURE_STATUS;

    /**
     * Generate a capture for the checkout with th given $checkoutId.
     * The payload data has to be conform with the paydirekt api data.
     * This function filters the 'action' and 'checkoutId' fields.
     *
     * The payloadData will be parsed to json and pushed to the Utils::runCurl function
     *
     * response: the paydirekt response
     * http_code: the http status code
     *
     * header: the full response from curl_getinfo()
     *
     * @param string $token - the token, created by paydirekt api
     * @param string $checkoutId - the paydirekt checkoutId
     * @param array $payloadData - the php array with all data, needed by this paydirekt api method
     * @return array
     */
    public static function createCapture($token, $checkoutId, $payloadData, $liveSystem)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);
        $payload = array();
        foreach ($payloadData as $key => $value) {
            if (!empty($value) && ($key <> "action") && ($key <> static::PAYDIREKT_CHECKOUTID_KEY)) {
                $payload[$key] = $value;
            }
        }
        $payload = json_encode($payload);
        if ($liveSystem === true) {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_LIVE;
        } else {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX;
        }
        $endpoint = $environment . "/" . $checkoutId . "/captures";
        return Utils::runCurl(Utils::getDefaultHeader($token), $payload, $endpoint, true);
    }

    /**
     * Returns the data of the specific capture.
     * The capture transactions are embedded in the checkout-resource
     *
     * response: the paydirekt response
     * http_code: the http status code
     *
     * header: the full response from curl_getinfo()
     *
     * @param string $token - the token, created by paydirekt api
     * @param string $checkoutId - the paydirekt checkoutId
     * @param string $captureId - the capture specific captureId
     * @return array
     */
    public static function retrieveCapture($token, $checkoutId, $captureId, $liveSystem)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);
        if ($liveSystem === true) {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_LIVE;
        } else {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX;
        }
        $endpoint = $environment . "/" . $checkoutId . "/captures/" . $captureId;
        return Utils::runCurl(Utils::getDefaultHeader($token), false, $endpoint, false);
    }

    /**
     * Closes the checkout/order with the given checkoutId
     * response: the paydirekt response
     *
     * http_code: the http status code
     *
     * header: the full response from curl_getinfo()
     *
     * @param string $token the token, created by paydirekt api
     * @param string $checkoutId the paydirekt checkoutId
     * @return array
     */
    public static function closeOrder($token, $checkoutId, $liveSystem)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);
        if ($liveSystem === true) {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_LIVE;
        } else {
            $environment = self::PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX;
        }
        $endpoint = $environment . "/" . $checkoutId . "/close";
        return Utils::runCurl(Utils::getDefaultHeader($token), false, $endpoint, true);
    }
}
