<?php
/**
 * Checkout Class
 * All nessesary functions for the checkout
 * or retrieving checkout informations
 * 
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.4
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\Lib;

class Checkout
{
    const PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX = "https://api.sandbox.paydirekt.de/api/checkout/v1/checkouts";
    const PAYDIREKT_CHECKOUT_ENDPOINT_LIVE = "https://api.paydirekt.de/api/checkout/v1/checkouts";
    const PAYDIREKT_TRANSACTIONID_KEY = "transactionId";
    const PAYDIREKT_STATUS_KEY = "status";
    const PAYDIREKT_TOKEN_KEY = "token";
    const PAYDIREKT_SHOPPING_BASKET_KEY = "shoppingBasket";
    const PAYDIREKT_ACTION_KEY = "action";
    const PAYDIREKT_UTILS_INCLUDE = "utils.php";
    const PAYDIREKT_CAPTURE_AMOUNT_EXEEDED = "CAPTURE_AMOUNT_EXCEEDED";

    const PAYDIREKT_MULTIREASON_KEY = "multireason";
    const PAYDIREKT_CODE_KEY = "code";
    const PAYDIREKT_REASON_KEY = "reason";
    const PAYDIREKT_MESSAGE_KEY = "message";
    const PAYDIREKT_DESCRIPTION_KEY = "description";

    const STATUS = CHECKOUT_STATUS;

    /**
     * Returns a 'unique' md5 hash
     *
     * @return string md5 hash
     */
    public static function getHash()
    {
        return md5(uniqid('', true));
    }

    /**
     * Returns a md5 hash, that includes the amount and the customerId
     *
     * @param string $amount the amount for hashing
     * @param string $customerId the customerId for hashing
     * @return string md5 hash
     */
    public static function createPaymentToken($amount, $customerId)
    {
        return md5(implode('|', array($amount, $customerId)));
    }

    /**
     * deprecated since 1.0.4
     *
     * @deprecated since 1.0.4
     * @param string $name the key for searching
     * @param string $standard the default value, if the key is not set
     * @return object|string the found value|the given standard value
     */
    public static function getParam($name = "GET_OR_POST_PARAM", $standard = "")
    {
        trigger_error("Deprecated function called.", E_USER_NOTICE);
        return null;
    }

    /**
     * Returns an empty PaydirektResponse Object.
     *
     * @param array $request
     * @return PaydirektResponse
     */
    public static function createPaymentResponse($request)
    {
        require_once('paydirektresponse.php');
        $response = new PaydirektResponse();
        $response->transactionId = $request->getParam(static::PAYDIREKT_TRANSACTIONID_KEY, null);
        $response->status = $request->getParam(static::PAYDIREKT_STATUS_KEY, null);
        $response->token = $request->getParam(static::PAYDIREKT_TOKEN_KEY, null);
        return $response;
    }

    /**
     * Reformat the parameter string.
     * When the parameter string has more than 20 characters,
     * it will be cropped to 20 characters.
     *
     * Allowed: [a-Z], [0-9], [':?,-(+.)/]
     * Max-size: 20 characters
     *
     * @param string The formattable string
     * @return string The formatted string
     */
    public static function getValidSepaString($string)
    {
        $replaces = preg_replace('/[a-z]|[0-9]|[\'\:\?\,\-\(\+\.\)\/]/i', '', $string);
        if ($replaces) {
            for ($i=0; $i < strlen($replaces); $i++) {
                $string = str_ireplace(substr($replaces, $i, 1), '?', $string);
            }
        }
        return substr($string, 0, 20);
    }
    public static function getValidSepaStringNote($string)
    {
        $replaces = preg_replace('/[a-z]|[0-9]|[\'\:\?\,\-\(\+\.\)\/]/i', '', $string);
        if ($replaces) {
            for ($i=0; $i < strlen($replaces); $i++) {
                $string = str_ireplace(substr($replaces, $i, 1), ' ', $string);
            }
        }
        return substr($string, 0, 37);
    }

    /**
     * Returns a filtered item array as json string representation.
     * If there is no valid item in the items an error string will be returned.
     *
     * @param array Items from basket
     * @return string
     */
    public static function basketItems($items)
    {
        $returnValue = null;

        if (!isset($_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY])) {
            $_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY] = array();
        }
        $item = array();

        foreach ($items as $key => $value) {
            if (!empty($value) && ($key <> static::PAYDIREKT_ACTION_KEY)) {
                $item[$key] = $value;
            }
        }
        if (!empty($item)) {
            array_push($_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY], $item);
            $returnValue = json_encode($_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY]);
        } else {
            $returnValue = "Keine Items";
        }
        return $returnValue;
    }

    /**
     * Sending all Data to the paydirekt api and returns the response.
     *
     * @param string $token
     * @param array $shippingAddress
     * @param array $payloadHead
     * @param array $payloadEnd
     * @param boolean $liveSystem
     * @return void
     */
    public static function createCheckout($token, $shippingAddress, $payloadHead, $payloadEnd, $liveSystem)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);
        $payload = array();
        foreach ($payloadHead as $key => $value) {
            if (!empty($value) && ($key <> static::PAYDIREKT_ACTION_KEY)) {
                $payload[$key] = $value;
            }
        }
        $payload["items"] = $_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY];
        unset($_SESSION[static::PAYDIREKT_SHOPPING_BASKET_KEY]);
        $payload["shippingAddress"] = $shippingAddress;

        foreach ($payloadEnd as $key => $value) {
            if (!empty($value) && ($key <> static::PAYDIREKT_ACTION_KEY)) {
                $payload[$key] = $value;
            }
        }

        $payload = json_encode($payload);

        if ($liveSystem === true) {
            $Environment = static::PAYDIREKT_CHECKOUT_ENDPOINT_LIVE;
        } else {
            $Environment = static::PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX;
        }
        return Utils::runCurl(Utils::getDefaultHeader($token), $payload, $Environment, true);
    }
    /**
     * Returns a complete checkout for the given $checkoutId parameter.
     *
     * @param array $token
     * @param string $checkoutId
     * @param boolean $liveSystem
     * @return void
     */
    public static function retrieveCheckout($token, $checkoutId, $liveSystem)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);
        if ($liveSystem === true) {
            $Environment = static::PAYDIREKT_CHECKOUT_ENDPOINT_LIVE;
        } else {
            $Environment = static::PAYDIREKT_CHECKOUT_ENDPOINT_SANDBOX;
        }
        $endpoint = $Environment . "/" . $checkoutId;
        return Utils::runCurl(Utils::getDefaultHeader($token), false, $endpoint, false);
    }
}
