<?php
/**
 * PaydirektLogger Class
 *
 * Usage:
 * $PaydirektLogger = PaydirektLogger::getInstance(get_class($this), $configuredLogLevel)
 * $PaydirektLogger->debug("This is a debug message!");
 * $PaydirektLogger->info("This is an array", array("hello" => "world"));
 * $PaydirektLogger->error("This is an error message!", array("ERROR" => "Not authorized"));
 *
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.4
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\Lib;

class PaydirektLogger
{

    const PAYDIREKT_STATUS_ERROR = "ERROR";
    const PAYDIREKT_STATUS_INFO = "INFO";
    const PAYDIREKT_STATUS_DEBUG = "DEBUG";
    const PAYDIREKT_STATUS_DEAKTIVIERT = "DEAKTIVIERT";

    protected static $instance = null;

    private $className = "";
    private $maxLogLevel = self::PAYDIREKT_STATUS_ERROR;
    private $fileName = "paydirekt.log";
    private $rotation = "DAILY"; // options: DAILY, WEEKLY, MONTHLY
    private $logFolder = "/log/";

    /**
     * Empty constructor, because this is a singleton pattern style class
     */
    protected function __construct()
    {
    }

    public static function getInstance($class, $maxLogLevel = self::PAYDIREKT_STATUS_ERROR)
    {
        static $instance = null;
        if ($instance === null) {
            $instance = new PaydirektLogger();
        }
        $instance->setClassName($class);
        $instance->setMaxLogLevel($maxLogLevel);

        return $instance;
    }

    public function debug($message, $data = false, $responseObj = false)
    {
        $this->log(self::PAYDIREKT_STATUS_DEBUG, $message, $this->className, $data, false, __DIR__, $responseObj);
    }
    public function error($message, $data = false, $responseObj = false)
    {
        $this->log(self::PAYDIREKT_STATUS_ERROR, $message, $this->className, $data, false, __DIR__, $responseObj);
    }
    public function info($message, $data = false, $responseObj = false)
    {
        $this->log(self::PAYDIREKT_STATUS_INFO, $message, $this->className, $data, false, __DIR__, $responseObj);
    }

    public function readLog($logLevel = false, $caller = false, $time = false, $json = false, $latest = false)
    {
        $logContent = $this->readFromLog(__DIR__, $this->logFolder, $this->fileName, $latest);
        $logEntries = explode("\n", $logContent);
        $logArray = array();
        $theMessage = "";
        $theTime = null;
        $theLevel = null;
        $theClass = null;
        foreach ($logEntries as $logEntry) {
            if ($this->startsWith($logEntry, "[")) {
                if ($theMessage != "") {
                    $logArray[] = array(
                        "time" => str_replace(array("[", "]"), "", $theTime),
                        "level" => str_replace(array("[", "]"), "", $theLevel),
                        "class" => str_replace(array("[", "]"), "", $theClass),
                        "message" => str_replace('"', "'", $theMessage),
                        "logid" => count($logArray),
                    );
                }

                $theMessage = "";
                $logContentArray = explode("::", $logEntry);

                $theTime = $logContentArray[0];
                $theLevel = $logContentArray[1];
                $theClass = $logContentArray[2];
                $theMessage = $logContentArray[3];

                // add filter, if given
                if ($logLevel !== false && strpos($theLevel, $logLevel) === false) {
                    continue;
                }
                if ($caller !== false && strpos($theClass, $caller) === false) {
                    continue;
                }
                if ($time !== false && strpos($theTime, $time) === false) {
                    continue;
                }
            } else {
                $theMessage .= $logEntry;
                continue;
            }
        }

        $logArray[] = array(
            "time" => str_replace(array("[", "]"), "", $theTime),
            "level" => str_replace(array("[", "]"), "", $theLevel),
            "class" => str_replace(array("[", "]"), "", $theClass),
            "message" => str_replace('"', "'", $theMessage),
            "logid" => count($logArray),
        );

        return $json !== false ? json_encode($logArray) : $logArray;
    }

    private function log($logLevel, $message, $caller = false, $data = false, $fileName = false, $rootFoldername = __DIR__, $responseObj = false)
    {
        if (!$this->hasMaxLogLevel($logLevel)) {
            return;
        }

        if ($data !== false) {
            $message .= var_export(PaydirektLogger::export($data, 5), true);
        }

        if ($responseObj !== false) {
            $message .= var_export(PaydirektLogger::export($responseObj, 5), true);
        }

        if ($fileName === false) {
            $fileName = $this->fileName;
        }
        $format = "[%s]::[%s]::[%s]::%s";
        $logString = sprintf($format, gmdate("d.m.Y H:i:s", time()), $logLevel, $caller, $message);
        $this->writeToLog($logString, $fileName, $rootFoldername);
    }

    private function hasMaxLogLevel($logLevel)
    {
        $theMaxLogLevel = $this->maxLogLevel;
        $returnValue = false;
        switch ($logLevel) {
            case self::PAYDIREKT_STATUS_DEBUG:
                $returnValue = ($theMaxLogLevel == self::PAYDIREKT_STATUS_DEBUG);
                break;
            case self::PAYDIREKT_STATUS_INFO:
                $returnValue = ($theMaxLogLevel == self::PAYDIREKT_STATUS_DEBUG
                    || $theMaxLogLevel == self::PAYDIREKT_STATUS_INFO);
                break;
            case self::PAYDIREKT_STATUS_ERROR:
                $returnValue = ($theMaxLogLevel == self::PAYDIREKT_STATUS_DEBUG
                    || $theMaxLogLevel == self::PAYDIREKT_STATUS_INFO
                    || $theMaxLogLevel == self::PAYDIREKT_STATUS_ERROR);
                break;
            case self::PAYDIREKT_STATUS_DEAKTIVIERT:
            default:
                $returnValue = false;
                break;
        }

        return $returnValue;
    }

    private function writeToLog($message, $fileName, $rootFoldername)
    {
        if (!is_dir($rootFoldername . $this->logFolder)) {
            $folderPermission = fileperms($rootFoldername);
            mkdir($rootFoldername . $this->logFolder, $folderPermission, true);
            file_put_contents($rootFoldername . $this->logFolder . $fileName, "");
            chmod($rootFoldername . $this->logFolder . $fileName, $folderPermission);
            file_put_contents($rootFoldername . $this->logFolder . ".htaccess", "<Files '*.log'>\nOrder Allow,Deny\nDeny from all\n</Files>");
            chmod($rootFoldername . $this->logFolder . ".htaccess", $folderPermission);
        }
        $message .= "\n";

        if ($this->rotation !== false) {
            switch ($this->rotation) {
                case "WEEKLY":
                    $timeformat = "W-Y";
                    break;
                case "MONTHLY":
                    $timeformat = "m-Y";
                    break;
                case "DAILY":
                default:
                    $timeformat = "d-m-Y";
                    break;
            }
            $ext = pathinfo($fileName, PATHINFO_EXTENSION);
            $fileNameSplit = explode($ext, $fileName);
            if (count($fileNameSplit) > 1) {
                $fileName = str_replace(".", "_", $fileNameSplit[0]);
                $fileName .= gmdate($timeformat, time());
                $fileName .= "." . $ext;
            }
        }

        return file_put_contents($rootFoldername . $this->logFolder . $fileName, $message, FILE_APPEND);
    }

    private function readFromLog($rootFoldername, $logFolder, $fileName, $latest = false)
    {
        $retVal = false;
        if ($latest) {
            $fileName = $this->getLastLog($rootFoldername, $logFolder);
        }

        if (is_dir($rootFoldername . $logFolder)) {
            $retVal = file_get_contents($rootFoldername . $logFolder . $fileName);
        }
        return $retVal;
    }

    private function getLastLog($rootFoldername, $logFolder)
    {
        $retVal = false;
        $files = scandir($rootFoldername . $logFolder, SCANDIR_SORT_DESCENDING);
        if ($files && count($files) > 0) {
            $retVal = $files[0];
        }

        return $retVal;
    }

    private function startsWith($haystack, $needle)
    {
        $length = strlen($needle);
        return (substr($haystack, 0, $length) === $needle);
    }

    public static function export($var, $maxDepth)
    {
        $return = null;
        $isObj = is_object($var);
        if ($var instanceof Collection) {
            $var = $var->toArray();
        }
        if (!$maxDepth) {
            return is_object($var) ? get_class($var)
            : (is_array($var) ? 'Array(' . count($var) . ')' : $var);
        }
        if (is_array($var)) {
            $return = array();
            foreach ($var as $k => $v) {
                $return[$k] = static::export($v, $maxDepth - 1);
                if (!PaydirektLogger::isMemoryAvailable()) {
                    break;
                }
            }
            return $return;
        }
        if (!$isObj) {
            return $var;
        }
        $return = new \stdclass();
        if ($var instanceof \DateTimeInterface) {
            $return->__CLASS__ = get_class($var);
            $return->date = $var->format('c');
            $return->timezone = $var->getTimezone()->getName();
            return $return;
        }

        $return->__CLASS__ = get_class($var);
        if ($var instanceof Proxy) {
            $return->__IS_PROXY__ = true;
            $return->__PROXY_INITIALIZED__ = $var->__isInitialized();
        }

        if ($var instanceof \ArrayObject || $var instanceof \ArrayIterator) {
            $return->__STORAGE__ = static::export($var->getArrayCopy(), $maxDepth - 1);
        }
        return self::fillReturnWithClassAttributes($var, $return, $maxDepth);
    }
    /**
     * Fill the $return variable with class attributes
     * Based on obj2array function from {@see http://php.net/manual/en/function.get-object-vars.php#47075}
     *
     * @param object   $var
     * @param \stdClass $return
     * @param int      $maxDepth
     *
     * @return mixed
     */
    private static function fillReturnWithClassAttributes($var, \stdClass $return, $maxDepth)
    {
        $clone = (array) $var;
        foreach (array_keys($clone) as $key) {
            $aux = explode("\0", $key);
            $name = end($aux);
            if ($aux[0] === '') {
                $name .= ':' . ($aux[1] === '*' ? 'protected' : $aux[1] . ':private');
            }
            $return->$name = static::export($clone[$key], $maxDepth - 1);

            if (!PaydirektLogger::isMemoryAvailable()) {
                break;
            }
        }
        return $return;
    }
    private static function isMemoryAvailable()
    {
        $memoryLimit = ini_get('memory_limit');
        $memoryLimit = str_replace("G","000", $memoryLimit);
        if (empty($memoryLimit)) {
            $memoryLimit = 64;
        }
        if ((((str_replace('M', '', $memoryLimit) * 1024 * 1024)) * 0.70 > memory_get_usage())) {
            return true;
        } else {
            return false;
        }
    }

    /*      GETTER / SETTER        */
    public function setClassName($className)
    {
        $this->className = $className;
    }
    public function setMaxLogLevel($maxLogLevel)
    {
        $this->maxLogLevel = $maxLogLevel;
    }
    public function setFileName($fileName)
    {
        $this->fileName = $fileName;
    }
    public function setRotation($rotation)
    {
        $this->rotation = $rotation;
    }
    public function disableRotation()
    {
        $this->rotation = false;
    }
    public function setLogFolder($logFolder)
    {
        $this->logFolder = $logFolder;
    }
}
