<?php
/**
 * Token class
 *
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.4
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\Lib;
use Bitergo\PaydirektWooCommerce\Lib\Utils;

class Token
{
    const PAYDIREKT_TOKEN_OBTAIN_ENDPOINT_SANDBOX = "https://api.sandbox.paydirekt.de/api/merchantintegration/v1/token/obtain";
    const PAYDIREKT_TOKEN_OBTAIN_ENDPOINT_LIVE = "https://api.paydirekt.de/api/merchantintegration/v1/token/obtain";
    const PAYDIREKT_TOKENEXPIRY_KEY = "token_expiry";
    const PAYDIREKT_UTILS_INCLUDE = "utils.php";
    const PAYDIREKT_RESPONSEKEY_KEY = "response";
    const PAYDIREKT_ACCESSTOKEN_KEY = "access_token";
    const PAYDIREKT_APIKEY_KEY = "apiKey";

    const PAYDIREKT_CODE_KEY = "code";
    const PAYDIREKT_REASON_KEY = "reason";
    const PAYDIREKT_MESSAGE_KEY = "message";
    const PAYDIREKT_DESCRIPTION_KEY = "description";
    const PAYDIREKT_UNAUTHORIZED_KEY = "Unauthorized";


    const STATUS = TOKEN_STATUS;


    private static $error = null;

    /**
     * Returns a token, needed for paydirekt communication.
     * If the token is expired, it automatically creates a new one.
     * There is also a threashold parameter, that checks if the token will be expired in "n" seconds
     *
     * @param string $configApiSecret
     * @param string $configApiKey
     * @param boolean $liveLiveSystem
     * @return array
     */
    public static function getToken($configApiSecret, $configApiKey, $liveSystem, $seconds = 600)
    {
        require_once(static::PAYDIREKT_UTILS_INCLUDE);

        // invalidate token, if api key changed
        if(isset($_SESSION[static::PAYDIREKT_APIKEY_KEY]) && $configApiKey != $_SESSION[static::PAYDIREKT_APIKEY_KEY]){
            unset($_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY]);
            $_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY] = array();
        }

        if (static::checktokenexpiry($seconds)) {
            $requestID = Utils::createUUID();
            $randomNonce = Utils::nonce();
            $now = time();
            $timestamp = gmdate("YmdHis", $now);
            $signature = Utils::hashSignature($requestID, $configApiKey, $timestamp, $randomNonce, $configApiSecret);
            $timestamp = gmdate(DATE_RFC1123, $now);

            $header = array();
            array_push($header, "X-Auth-Key: " .$configApiKey);
            array_push($header, "X-Auth-Code: " .$signature);
            array_push($header, "X-Request-ID: " .$requestID);
            array_push($header, "X-Date: " .$timestamp);
            array_push($header, "Content-Type: application/hal+json;charset=utf-8");
            array_push($header, "Accept: application/hal+json");

            $body = array(
                "grantType" => "api_key",
                "randomNonce" => $randomNonce
            );
            $body = json_encode($body);

            if ($liveSystem === true) {
                $environment = self::PAYDIREKT_TOKEN_OBTAIN_ENDPOINT_LIVE;
            } else {
                $environment = self::PAYDIREKT_TOKEN_OBTAIN_ENDPOINT_SANDBOX;
            }

            $tokenresponse = Utils::runCurl($header, $body, $environment, true);
            $returnValue = null;

            if ($tokenresponse['http_code'] != 200) {
                $status = Utils::resolveStatus(self::STATUS, $tokenresponse);
                $_SESSION[static::PAYDIREKT_ACCESSTOKEN_KEY] = "";
                static::setError($status);
            } else {
                $token = json_decode($tokenresponse[static::PAYDIREKT_RESPONSEKEY_KEY], true);
                $_SESSION[static::PAYDIREKT_ACCESSTOKEN_KEY] = $token[static::PAYDIREKT_ACCESSTOKEN_KEY];
                $_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY] = time()+$token["expires_in"];
                $_SESSION[static::PAYDIREKT_APIKEY_KEY] = $configApiKey;
                $returnValue = $_SESSION[static::PAYDIREKT_ACCESSTOKEN_KEY];
                static::setError(null);
            }
        }else{
            $returnValue = $_SESSION[static::PAYDIREKT_ACCESSTOKEN_KEY];
        }
        return $returnValue;
    }

    /**
     * Checks if the token will expire in "n" seconds.
     * Returns true, if the token will be expire in given seconds, otherwise false.
     *
     * @return boolean
     */
    public static function checktokenexpiry($seconds)
    {
        $expired = true;
        if (isset($_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY]) 
        && !empty($_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY]) 
        && ($_SESSION[static::PAYDIREKT_TOKENEXPIRY_KEY]-time()) >= $seconds) {
                $expired = false;
        }
        return $expired;
    }

    /**
     * gets the error
     *
     * @return void
     */
    public static function getError()
    {
        return self::$error;
    }
    /**
     * sets the error
     *
     * @param array $error
     * @return void
     */
    public static function setError($error)
    {
        self::$error = $error;
    }
}
