<?php
/**
 * Utils class with helpers to work with the paydirekt api
 *
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.4
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\Lib;
use Bitergo\PaydirektWooCommerce\Lib\PaydirektLogger;

class Utils
{

    const PAYDIREKT_CODE_KEY = "code";
    const PAYDIREKT_REASON_KEY = "reason";
    const PAYDIREKT_MESSAGE_KEY = "message";
    const PAYDIREKT_DESCRIPTION_KEY = "description";
    const PAYDIREKT_UNPROCESSABLE_ENTITIY = "Unprocessable Entity";
    const PAYDIREKT_HTTP_CODE = "http_code";
    const PAYDIREKT_PAYDIREKT_CODE = "payDirekt";
    const PAYDIREKT_CHECKOUTS_CODE = "checkouts";
    const PAYDIREKT_MULTIREASON_CODE = "multireason";

    const PAYDIREKT_PLUGIN_ID = "paydirekt_";
    const PAYDIREKT_SHOP_ID_KEY = 'shop_id';
    const PAYDIREKT_SHOP_VERSION_KEY = 'shop_version';
    const PAYDIREKT_SHOP_VERSIONBUILD_KEY = 'shop_version_build';
    const PAYDIREKT_SHOP_SHORTNAME_KEY = 'shop_short_name';

    const STATUS = UTILS_STATUS;

    public static $isWordpressSite = null;

    /**
     * Returns a randomBytes string.
     *
     * @param int $length
     * @return string
     */
    public static function randomBytes($length)
    {
        $retval = "";
        if ($length <= 0) {
            throw new \InvalidArgumentException("length is not a positive integer");
        }
        if (function_exists('openssl_random_pseudo_bytes')) {
            $retval = (openssl_random_pseudo_bytes($length));
        }
        if (function_exists('random_bytes')) {
            $retval = (random_bytes($length));
        }
        if (version_compare(PHP_VERSION, '7.0.0') <= 0) {
            if (function_exists('mcrypt_create_iv')) {
                $retval = (mcrypt_create_iv($length, MCRYPT_DEV_URANDOM));
            }
        }
        return $retval;
    }

    /**
     * Returns a 16 bit Universally Unique Identifier (UUID).
     *
     * @return string
     */
    public static function createUUID()
    {
        $input = static::randomBytes(16);
        $input = str_split(bin2hex($input), 4);
        $input = vsprintf("%s%s-%s-%s-%s-%s%s%s", $input);
        return $input;
    }

    /**
     * Encodes a base64 string and return the given value as base64 string.
     *
     * @param string $input
     * @return void
     */
    public static function encode64($input)
    {
        return strtr(base64_encode($input), "/+", "_-");
    }

    /**
     * Decodes a base64 string and return its value.
     *
     * @param string $input
     * @return string
     */
    public static function decode64($input)
    {
        return base64_decode(strtr($input, '-_', '+/'));
    }

    /**
     * Generates a 48bit nonce, that is encoded with base64.
     *
     * @return string
     */
    public static function nonce()
    {
        $nonce = static::randomBytes(48);
        return static::encode64($nonce);
    }

    /**
     * Generates a sign from given parameters
     *
     * @param string $requestID
     * @param int $timestamp
     * @param string $APIKEY
     * @param string $randomNonce
     * @return string
     */
    public static function stringToSign($requestID, $timestamp, $APIKEY, $randomNonce)
    {
        $stringtosign = $requestID .":" .$timestamp .":" .$APIKEY .":" .$randomNonce;
        return $stringtosign;
    }

    /**
     * Generates a hash signature
     *
     * @param string $requestID
     * @param string $APIKEY
     * @param int $timestamp
     * @param string $randomNonce
     * @param string $APISECRET
     * @return boolean
     */
    public static function hashSignature($requestID, $APIKEY, $timestamp, $randomNonce, $APISECRET)
    {
        $stringtosign = static::stringToSign($requestID, $timestamp, $APIKEY, $randomNonce);
        $apiSecretDecoded = static::decode64($APISECRET);
        $hash = hash_hmac("sha256", $stringtosign, $apiSecretDecoded, true);
        return static::encode64($hash);
    }

    /**
     * The main function for Wordpress to communicate with the paydirekt api server
     * It returns an array with all return values from the response.
     * 
     * $returnValue['response'] => the response object
     * $returnValue['http_code'] => the 'http_code' from the response header
     * $returnValue['header'] => some additional informations
     *
     * @param string $header
     * @param array $body
     * @param string $endpoint
     * @param boolean $post
     * @return array
     * 
     */
    public static function runHTTPWP($header, $body, $endpoint, $post){
        require_once("logger.php");
        $logger = PaydirektLogger::getInstance(get_class());

        $headers = array();
        foreach($header as $h){
            $explodedVar = explode(':', $h, 2); 
            $headers[$explodedVar[0]] = ltrim($explodedVar[1]);
        }

        if(!$post){
            $response = wp_remote_get($endpoint, array(
                'method'      => 'GET',
                'timeout'     => 10,
                'httpversion' => '1.1',
                'headers'     => $headers,
            ));

        } else {
            $response = wp_remote_post( $endpoint, array(
                'method'      => 'POST',
                'timeout'     => 10,
                'httpversion' => '1.1',
                'headers'     => $headers,
                'body'        => $body,
            ));
        }

        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            $logger->error("Error bei HTTP Anfrage: ", $error_message);
        } else {
            $returnValue['response'] = wp_remote_retrieve_body($response);
            $returnValue['http_code'] = wp_remote_retrieve_response_code($response);
            $returnValue['header'] = wp_remote_retrieve_headers($response);
            return $returnValue;
        }
        return null;
    }

    /**
     * The main function that decides wich request method to use based on the Shop System
     *
     * @param string $header
     * @param array $body
     * @param string $endpoint
     * @param boolean $post
     * @return array
     * 
     */
    public static function runCurl($header, $body, $endpoint, $post){
        return Utils::runHTTPWP($header, $body, $endpoint, $post);
    }


    /**
     * stores a value with the given hash to the session "payDirekt" namespace
     *
     * @param string $hash
     * @param string $key
     * @param mixed $value
     * @return void
     */
    public static function setSessionValuesByHash($hash, $key, $value)
    {
        $_SESSION[static::PAYDIREKT_PAYDIREKT_CODE][static::PAYDIREKT_CHECKOUTS_CODE][$hash][$key] = $value;
    }

    /**
     * returns a value with the given hash from the session "payDirekt" namespace
     *
     * @param string $hash
     * @return mixed
     */
    public static function getSessionValuesByHash($hash)
    {
        return isset($_SESSION[static::PAYDIREKT_PAYDIREKT_CODE][static::PAYDIREKT_CHECKOUTS_CODE][$hash])
            ? $_SESSION[static::PAYDIREKT_PAYDIREKT_CODE][static::PAYDIREKT_CHECKOUTS_CODE][$hash]
            : false;
    }

    /**
     * old Logging function
     * @deprecated 1.0.2 replaced by Logger class
     * @param string $message
     * @param string $mode
     * @return void
     */
    public static function logging($message, $mode = "DEBUG")
    {
        if($mode != "IR"){
            return file_put_contents("reports.txt", $message . "\r\n", FILE_APPEND);
        }
    }

    /**
     * Returns the status array for the status given as $status parameter.
     *
     * 1.
     * possible array types: <br/>
     * "200" => array(
     *          "code" => 200,
     *          "reason" => "All ok",
     *          "message" => "OK",
     *          "description" => "My description for output")
     *
     * 2.
     * "200" => array("code" => 200, "multireason" => array(
     *   "OK" => array("reason" => "All ok", "message" => "OK", "description" => "All is ok"),
     *   "ERROR_NOT_READY" => array(
     *          "reason" => "Not ready",
     *          "message" => "ERROR_NOT_READY",
     *          "description" => "Not all is ok")
     * )
     *
     * @param array $statusArray
     * @param array $response
     * @return array
     */
    public static function resolveStatus($statusArray, $response)
    {
        $statusArray = json_decode($statusArray, true);
        // the keys from the first array has priority and will not overwritten by the second array.
        $statusArray += json_decode(static::STATUS, true);

        if (!isset($statusArray) || !isset($response)) {
            throw new \InvalidArgumentException ('invalid arguments or arguments are null');
        }
        $returnValue = array();

        $responseMessage = json_decode($response['response'], true);
        if(isset($responseMessage['messages']) && isset($responseMessage['messages'][0])){
        	$firstMessage = $responseMessage['messages'][0];
        }
        $status = $response[static::PAYDIREKT_HTTP_CODE];

        if ($responseMessage && array_key_exists("messages", $responseMessage)) {
            $returnValue = $firstMessage;
        }

        if (array_key_exists($status, $statusArray)) {
            $returnValue = array_merge($returnValue, $statusArray[$status]);
            if (array_key_exists(static::PAYDIREKT_MULTIREASON_CODE, $statusArray[$status])) {
                unset($returnValue[static::PAYDIREKT_MULTIREASON_CODE]);
                $returnValue = array_merge($returnValue, $statusArray[$status][static::PAYDIREKT_MULTIREASON_CODE][$firstMessage['code']]);
            }
        }

        return $returnValue;
    }

    /**
     * Returns the default header for all api calls
     *
     * @throws \InvalidArgumentException if the argument is not set
     *
     * @param string $token
     * @return string
     */
    public static function getDefaultHeader($token)
    {
        if (!isset($token)) {
            throw new \InvalidArgumentException('token is null.');
        }

        $developerId = "com.fulino";
        $shopSystemId = "woocommerce";
        $pluginVersion = "1.0.44";
        $shopSystemVersion = "6.2.1";
        $shopSystemVersionBuild = "";
        $shopSystemShortName = "woo62";

        $envVars = static::getEnvironmentVars();
        !empty($envVars[static::PAYDIREKT_SHOP_ID_KEY])?$shopSystemId = $envVars[static::PAYDIREKT_SHOP_ID_KEY]:"";
        !empty($envVars[static::PAYDIREKT_SHOP_VERSION_KEY])?$shopSystemVersion = $envVars[static::PAYDIREKT_SHOP_VERSION_KEY]:"";
        !empty($envVars[static::PAYDIREKT_SHOP_VERSIONBUILD_KEY])?$shopSystemVersionBuild = $envVars[static::PAYDIREKT_SHOP_VERSIONBUILD_KEY]:"";

        $x_developer_id = "x-developer-id: " . $developerId;
        $x_shop_system_id = "x-shop-system-id: " . $shopSystemId;
        $x_shop_system_version = "x-shop-system-version: " . $shopSystemVersion;
        $x_plugin_id = "x-plugin-id: " . $shopSystemShortName;
        $x_plugin_version = "x-plugin-version: " . $pluginVersion;

        $timestamp = gmdate(DATE_RFC1123, time());

        $header = array();
        array_push($header, "Authorization: Bearer " .$token);
        array_push($header, "Content-Type: application/hal+json;charset=utf-8");
        array_push($header, "Accept: application/hal+json");
        array_push($header, "Date: " .$timestamp);
        array_push($header, $x_developer_id);
        array_push($header, $x_shop_system_id);
        array_push($header, $x_shop_system_version);
        array_push($header, $x_plugin_id);
        array_push($header, $x_plugin_version);

        return $header;
    }

    /**
     * Checks the apiKey for sandbox pattern. If the pattern matches, it returns false.
     * If the apiKey is empty, it throws an error
     *
     * @throws \InvalidArgumentException if the argument is not set
     * @param string $apiKey
     * @return boolean
     *
     */
    public static function isLiveSystem($apiKey)
    {
        if (!isset($apiKey)) {
            throw new \InvalidArgumentException('apiKey is null.');
        }

        $identConfigApiKey = substr($apiKey, 0, 8);
        $isLiveSystem = false;
        if($identConfigApiKey != '00000000') {
            $isLiveSystem = true;
        }

        return $isLiveSystem;
    }

    /**
     * checks a string to be compatible with the UUID4 standard
     *
     * @param string $value
     * @return mixed not found=0; found=1; error=false
     */
    public static function checkUUID4($value)
    {
        if (!isset($value)) {
            throw new \InvalidArgumentException('value is null.');
        }

        $regexStr = '/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/';
        return preg_match($regexStr, $value);
    }

    /**
     * checks a string to be compatible with the Base64 RFC4648 standard
     *
     * @param string $value
     * @return mixed not found=0; found=1; error=false
     */
    public static function checkBase64_RFC4648($value)
    {
        if (!isset($value)) {
            throw new \InvalidArgumentException('value is null.');
        }
        $regexStr = '/(?:^(?:[A-Za-z0-9_-]{4})*(?:[A-Za-z0-9_-]{2}==|[A-Za-z0-9_-]{3}=)$)/';
        return preg_match($regexStr, $value);
    }

    /**
     * check the apiKey parameter, if the format is correct.
     *
     * Format: is UUID4 & has 36 chars
     *
     * @param string $apiKey
     * @return boolean
     */
    public static function isApiKey($apiKey)
    {
        if (!isset($apiKey)) {
            throw new \InvalidArgumentException('apiKey is null.');
        }

        $status = false;
        if (static::checkUUID4($apiKey) === 1 && strlen($apiKey) == 36) {
            $status = true;
        }

        return $status;
    }

    /**
     * check the apiSecret parameter, if the format is correct
     *
     * Format: is Base64_RFC4668 & has 44 chars
     *
     * @param string $apiKey
     * @return boolean
     */
    public static function isApiSecret($apiSecret)
    {
        if (!isset($apiSecret)) {
            throw new \InvalidArgumentException('apiSecret is null.');
        }

        $status = false;
        if (static::checkBase64_RFC4648($apiSecret) === 1 && strlen($apiSecret) == 44) {
            $status = true;
        }

        return $status;
    }

    /**
     * check the minimumAge parameter, if the format is correct
     *
     * Format: is Numeric & is between 0 and 99
     *
     * @param string $apiKey
     * @return boolean
     */
    public static function isMinimumAge($minimumAge)
    {
        if (!isset($minimumAge)) {
            throw new \InvalidArgumentException('minimumAge is null.');
        }

        $status = false;
        if (is_numeric($minimumAge) && ($minimumAge<100 && $minimumAge > -1 )) {
            $status = true;
        }

        return $status;
    }

    public static function jsonValidate($string)
    {
        // decode the JSON data
        $result = json_decode($string);
        // switch and check possible JSON errors
        switch (json_last_error()) {
            case JSON_ERROR_NONE:
                $error = ''; // JSON is valid // No error has occurred
                break;
            case JSON_ERROR_DEPTH:
                $error = 'The maximum stack depth has been exceeded.';
                break;
            case JSON_ERROR_STATE_MISMATCH:
                $error = 'Invalid or malformed JSON.';
                break;
            case JSON_ERROR_CTRL_CHAR:
                $error = 'Control character error, possibly incorrectly encoded.';
                break;
            case JSON_ERROR_SYNTAX:
                $error = 'Syntax error, malformed JSON.';
                break;
            // PHP >= 5.3.3
            case JSON_ERROR_UTF8:
                $error = 'Malformed UTF-8 characters, possibly incorrectly encoded.';
                break;
            // PHP >= 5.5.0
            case JSON_ERROR_RECURSION:
                $error = 'One or more recursive references in the value to be encoded.';
                break;
            // PHP >= 5.5.0
            case JSON_ERROR_INF_OR_NAN:
                $error = 'One or more NAN or INF values in the value to be encoded.';
                break;
            case JSON_ERROR_UNSUPPORTED_TYPE:
                $error = 'A value of a type that cannot be encoded was given.';
                break;
            default:
                $error = 'Unknown JSON error occured.';
                break;
        }

        return $error === '';
    }

    public static function isISO8601Date($dateStr) {
            return (preg_match('/^([\+-]?\d{4}(?!\d{2}\b))((-?)((0[1-9]|1[0-2])(\3([12]\d|0[1-9]|3[01]))?|W([0-4]\d|5[0-2])(-?[1-7])?'
            . '|(00[1-9]|0[1-9]\d|[12]\d{2}|3([0-5]\d|6[1-6])))([T\s]((([01]\d|2[0-3])((:?)[0-5]\d)?|24\:?00)([\.,]\d+(?!:))?)'
            . '?(\17[0-5]\d([\.,]\d+)?)?([zZ]|([\+-])([01]\d|2[0-3]):?([0-5]\d)?)?)?)?$/', $dateStr) > 0);
    }

    /**
     * set shop id, shop version, shop build and a generated shortname to session vars
     *
     * @param string $shop_id
     * @param string $shop_version
     * @param string $shop_version_build
     * @param string $shop_short_name
     * @return void
     */
    public static function setEnvironmentVars($shop_id, $shop_version, $shop_version_build){
        $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_ID_KEY] = $shop_id;
        $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSION_KEY] = $shop_version;
        $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSIONBUILD_KEY] = $shop_version_build;
    }
    /**
     * get the session vars defined for this shop system
     *
     * @return array
     */
    public static function getEnvironmentVars(){
        $retval = array();
        if(isset($_SESSION[static::PAYDIREKT_PLUGIN_ID])){
            if(isset($_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_ID_KEY])){
                $retval[static::PAYDIREKT_SHOP_ID_KEY] = $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_ID_KEY];
            }else{
                $retval[static::PAYDIREKT_SHOP_ID_KEY] = "";
            }
            if(isset($_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSION_KEY])){
                $retval[static::PAYDIREKT_SHOP_VERSION_KEY] = $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSION_KEY];
            }else{
                $retval[static::PAYDIREKT_SHOP_VERSION_KEY] = "";
            }
            if(isset($_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSIONBUILD_KEY])){
                $retval[static::PAYDIREKT_SHOP_VERSIONBUILD_KEY] = $_SESSION[static::PAYDIREKT_PLUGIN_ID][static::PAYDIREKT_SHOP_VERSIONBUILD_KEY];
            }else{
                $retval[static::PAYDIREKT_SHOP_VERSIONBUILD_KEY] = "";
            }
            return $retval;
        }
    }
}
