<?php
/**
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.44
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\classes;

use Bitergo\PaydirektWooCommerce\classes\NotifyStatusEnum;
use Bitergo\PaydirektWooCommerce\Lib\PaydirektLogger;

if (!defined('ABSPATH')) {
    exit;
}
class PaydirektCallback
{

    /* the json decoded post data from the paydirekt api */
    private $postData;

    /* the order searchd by orderId */
    private $order;

    /* the transationId extracted from postData */
    private $transactionId;

    /* the checkout status extracted from postData */
    private $checkoutStatus;

    /* the merchantOrderReferenceNumber aka. orderno. extranced from postData */
    private $merchantOrderReferenceNumber;

    /* the paydirekt plugin settings */
    private $settings;

    /* the logger (core logger) */
    private $logger;

    /**
     * constructor
     *
     * @param Object $rawPostData
     * @param array $settings
     */
    public function __construct($rawPostData, $settings)
    {
        $this->logger = PaydirektLogger::getInstance(get_class($this), $settings[PAYDIREKT_PARAMETER_LOGLEVEL]);
        $this->postData = json_decode($rawPostData, true);
        $this->transactionId = $this->postData['checkoutId'];
        $this->checkoutStatus = $this->postData['checkoutStatus'];
        $this->merchantOrderReferenceNumber = $this->postData['merchantOrderReferenceNumber'];
        $this->order = new \WC_Order($this->merchantOrderReferenceNumber);
        $this->settings = $settings;
    }

    /**
     * processes the notify request, checks if the notify is valid and
     *
     * @return void
     */
    public function processNotify()
    {
        $this->logger->debug("processNotify start");
        if (!$this->isValidNotify()) {
            $this->logger->error("Notify not Valid!", $this->postData);
            return;
        }

        $this->logger->debug("processNotify checkout status: ", $this->checkoutStatus);
        switch ($this->checkoutStatus) {
            case NotifyStatusEnum::APPROVED:
                $this->logger->debug("processNotify oder payment complete. ", $this->transactionId);
                $this->order->payment_complete();
                break;
            case NotifyStatusEnum::REJECTED:
                $this->logger->debug("processNotify oder payment rejected. ", $this->transactionId);
            // there is no 'break'! That is correct, because it should jump into the canceled case.
            case NotifyStatusEnum::CANCELED:
                $this->logger->debug("processNotify oder payment canceled. ", $this->transactionId);
                $this->order->cancel_order();
                break;
            case NotifyStatusEnum::CLOSED:
                $this->logger->debug("processNotify oder payment closed. ", $this->transactionId);
                break;
            case NotifyStatusEnum::EXPIRED:
                $this->logger->debug("processNotify oder payment expired. ", $this->transactionId);
                $this->order->cancel_order();
                break;
            default:
                break;
        }

        $this->logger->debug("processNotify end");
    }

    /**
     * Checks if the Notify request is valid to protect against
     * manual or faulty requests
     *
     * - checks if the userId is equal to order userId
     * - checks if the request ip address is one of the paydirekt API addresses
     * - checks if the transactionId equals the order transactionId
     *
     * @return boolean
     */
    private function isValidNotify()
    {
        $this->logger->debug("isValidNotify start");

        $ipAddress = getenv('REMOTE_ADDR');
        $userId = sanitize_text_field($_GET['userId']);

        $this->logger->debug("isValidNotify end");
        // process checks, return true if all checks are successfull
        return (
            $this->isUserInOrder($userId) &&
            $this->isValidIpAdress($ipAddress) &&
            $this->isCheckoutIdValid()
        );
    }

    /**
     * checks if the userId equals the userId in the order
     *
     * @param string $userId
     * @return boolean
     */
    private function isUserInOrder($userId)
    {
        $this->logger->debug("isUserInOrder userId: ", $userId);
        return ((string) $this->order->get_user_id() === (string) $userId);
    }

    /**
     * checks if the remoteIpAddress matches one of the configured paydirekt API ip addresses
     *
     * @param string $remoteIpAddress
     * @return boolean
     */
    private function isValidIpAdress($remoteIpAddress)
    {
        $isIpAddress = false;
        $this->logger->debug("checkIpAdress: Start");
        $this->logger->debug("checkIpAdress: remoteIpAddress: ", $remoteIpAddress);
        if ($remoteIpAddress == PAYDIREKT_NOTIFY_SANDBOX_IPADRESS_KEY
            || $remoteIpAddress == PAYDIREKT_NOTIFY_LIVE_IPADRESS_KEY) {
            $this->logger->debug("checkIpAdress: End");
            $isIpAddress = true;
        }
        $this->logger->debug("checkIpAdress: MISMATCH IPADRESS: ", $remoteIpAddress);
        $this->logger->debug("checkIpAdress: End");
        return $isIpAddress;
    }
    /**
     * checks if the transactionId equals the transactionId in order
     *
     * @return boolean
     */
    private function isCheckoutIdValid()
    {
        return $this->order->get_transaction_id() === $this->transactionId;
    }

    public function getOrder()
    {
        return $this->order;
    }

    public function getTransactionId()
    {
        return $this->transactionId;
    }

    public function getCheckoutStatus()
    {
        return $this->checkoutStatus;
    }

    public function getMerchantOrderReferenceNumber()
    {
        return $this->merchantOrderReferenceNumber;
    }
}
