<?php
/**
 * @category    Payment Plugin
 * @package     paydirekt
 * @author      Logata Digital Solutions GmbH
 * @copyright   Logata Digital Solutions GmbH
 * @license     https://logata.com/paydirekt-lizenzbedingungen
 * @version     1.0.44
 * @link        https://www.paydirekt.de
 */

namespace Bitergo\PaydirektWooCommerce\classes;

use Bitergo\PaydirektWooCommerce\Lib\PaydirektLogger;
use Bitergo\PaydirektWooCommerce\classes\ResponseStatusEnum;
use Bitergo\PaydirektWooCommerce\classes\PaydirektUtils;
use Bitergo\PaydirektWooCommerce\Lib\Token;
use Bitergo\PaydirektWooCommerce\Lib\Utils;
use Bitergo\PaydirektWooCommerce\Lib\Checkout;

if (!defined('ABSPATH')) {
    exit;
}
class PaydirektCheckout
{

    private $order;
    private $error;
    private $logger;
    private $settings;
    private $redirectUrl;

    public function __construct($order)
    {
        $this->settings = PaydirektUtils::getSettings($order);
        $this->order = $order;
        $this->logger = PaydirektLogger::getInstance(get_class($this), $this->settings[PAYDIREKT_PARAMETER_LOGLEVEL]);
    }

    public function getOrder()
    {
        return $this->order;
    }
    public function setOrder($order)
    {
        $this->order = $order;
        $this->settings = PaydirektUtils::getSettings($order);
    }
    public function getError()
    {
        return $this->error;
    }
    public function getRedirectUrl()
    {
        return $this->redirectUrl;
    }

    public function getReturnUrl()
    {
        return \add_query_arg(array("woopaydirekt" => "true",
            "order_id" => $this->order->get_id()),
            $this->order->get_checkout_order_received_url()
        );
    }

    public function getNotifyUrl()
    {
        $notifyBaseUrl = \get_site_url() . "/wc-api/CALLBACK";
        $this->logger->debug("notify url: ", $notifyBaseUrl);
        \remove_query_arg("wc-ajax", $notifyBaseUrl);
        return $notifyBaseUrl;
    }

    public function createCheckout()
    {
        $this->logger->debug("createCheckout start");
        $apiKey = $this->settings[PAYDIREKT_PARAMETER_APIKEY];
        $apiSecret = $this->settings[PAYDIREKT_PARAMETER_APISECRET];
        if (empty($apiKey) || empty($apiSecret)) {
            return null;
        }

        try {
            $liveSystem = Utils::isLiveSystem($apiKey);
            $this->logger->debug("createCheckout is liveSystem", $liveSystem);
        } catch (\InvalidArgumentException $ex) {
            $this->error['description'] = "Der angegebene API Key ist nicht korrekt";
            $this->logger->error("API Key not correct.", $ex);
        }

        $token = Token::getToken($apiSecret, $apiKey, $liveSystem);
        if (Token::getError() != null) {
            $this->error = Token::getError();
            return null;
        }

        $this->fillCheckoutBasket();

        $customerShippingAddress = $this->buildCustomerShippingAddress();
        $payloadHead = $this->buildPayloadHead();
        $payloadEnd = $this->buildPayloadEnd();

        try {
            $postCheckout = Checkout::createCheckout($token, $customerShippingAddress, $payloadHead, $payloadEnd, $liveSystem);
        } catch (\InvalidArgumentException $ex) {
            $this->logger->error("token error", $ex);
            $errors[] = "Der Token konnte nicht generiert werden";
        }

        $checkoutResponse = json_decode($postCheckout['response'], true);

        try {
            $statusCode = Utils::resolveStatus(Checkout::STATUS, $postCheckout);
        } catch (\InvalidArgumentException $ex) {
            $this->logger->error("statusArray or response is not set", $ex);
        }

        if ($statusCode['message'] == "OK") {
            $this->logger->info("createCheckout: Checkout created successfully. ", $checkoutResponse['checkoutId']);
        } else {
            $this->logger->error("createCheckout: Checkout creating failed. ", $checkoutResponse);
        }

        if (empty($this->error)) {
            try {
                $this->redirectUrl = $checkoutResponse['_links']['approve']['href'];
                return $checkoutResponse;
            } catch (\Exception $e) {
                $this->logger->error("Unable to set status for order");
            }
        }

        $this->logger->debug("indexAction end");
    }

    /**
     * @param array $basket
     * @param array $basketType
     * @return array
     */
    private function buildPayloadHead()
    {
        $payloadHead = array();
        if ($this->settings[PAYDIREKT_PARAMETER_MINIMUMAGE] != null
            && $this->settings[PAYDIREKT_PARAMETER_MINIMUMAGE] > 0) {
            $payloadHead[PAYDIREKT_PARAMETER_MINIMUMAGE] = $this->settings[PAYDIREKT_PARAMETER_MINIMUMAGE];
        }

        $shippingTax = 0;

        if($this->order->get_shipping_tax() != null){
           $shippingTax = $this->order->get_shipping_tax();
        }

       $payloadHead[PAYDIREKT_TYPE] = $this->settings[PAYDIREKT_PARAMETER_CHOOSECHECKOUT];
       $payloadHead[PAYDIREKT_TOTAL_AMOUNT] = $this->order->get_total();
       $payloadHead[PAYDIREKT_SHIPPING_AMOUNT] = PAYDIREKT_COMPATIBILITY ? number_format(($this->order->get_total_shipping() + $shippingTax), 2, '.', '') : number_format(($this->order->get_shipping_total() + $shippingTax), 2, '.', '');
       $orderAmount = $this->order->get_total() - (
           PAYDIREKT_COMPATIBILITY ?
           $this->order->get_total_shipping() :
           $this->order->get_shipping_total()
       );
        $payloadHead[PAYDIREKT_ORDER_AMOUNT] = number_format($orderAmount, 2, '.', '');
        $payloadHead[PAYDIREKT_CURRENCY] = PAYDIREKT_COMPATIBILITY ? $this->order->get_order_currency() : $this->order->get_currency();
        $payloadHead[PAYDIREKT_SHOPPING_CART_TYPE] = $this->getBasketType();

        return $payloadHead;
    }

    /**
     * @param array $user
     * @param array $urlResponse
     * @param string $hash
     * @param string $shopwareToken
     * @param array $basketType
     * @return array
     */
    private function buildPayloadEnd()
    {
        $userId = PAYDIREKT_COMPATIBILITY ? $this->order->get_user_id() : $this->order->get_customer_id();
        $orderNumber = $this->order->get_order_number();
        $urlResponse = $this->getReturnUrl();
        $urlNotify = $this->getNotifyUrl();
        $hash = PaydirektUtils::getHash();
        $shopwareToken = Checkout::createPaymentToken($this->order->get_total(), $userId);

        $redirectUrlAfterSuccess = \add_query_arg(
            array(
                PAYDIREKT_STATUS => ResponseStatusEnum::ACCEPTED,
                PAYDIREKT_HASH => $hash,
                PAYDIREKT_TOKEN => $shopwareToken,
            ), $urlResponse
        );

        $redirectUrlAfterCancellation = $this->order->get_cancel_order_url();

        $redirectUrlAfterCancellation = \add_query_arg(
            array(
                PAYDIREKT_STATUS => ResponseStatusEnum::CANCEL,
                PAYDIREKT_HASH => $hash,
                PAYDIREKT_TOKEN => $shopwareToken,
            ), $urlResponse
        );
        $redirectUrlAfterRejection = \add_query_arg(
            array(
                PAYDIREKT_STATUS => ResponseStatusEnum::REJECT,
                PAYDIREKT_HASH => $hash,
                PAYDIREKT_TOKEN => $shopwareToken,
            ), $urlResponse
        );

        $redirectUrlAfterAgeVerificationFailure = \add_query_arg(
            array(
                PAYDIREKT_STATUS => ResponseStatusEnum::AGE,
                PAYDIREKT_HASH => $hash,
                PAYDIREKT_TOKEN => $shopwareToken,
            ), $urlResponse
        );

        $callbackUrlStatusUpdates = \add_query_arg(
            array(
                PAYDIREKT_ACTION => "paydirekt-notify",
                PAYDIREKT_ORDER_ID => $orderNumber,
                PAYDIREKT_HASH => $hash,
                PAYDIREKT_USER_ID => $userId,
            ), $urlNotify
        );

        return array(
            PAYDIREKT_MERCHANT_CUSTOMER_NUMBER => $userId,
            PAYDIREKT_MERCHANT_ORDER_REFERENCE_NUMBER => $orderNumber,
            PAYDIREKT_REDIRECTURL_AFTER_SUCCESS => $redirectUrlAfterSuccess,
            PAYDIREKT_REDIRECTURL_AFTER_CANCELLATION => $redirectUrlAfterCancellation,
            PAYDIREKT_REDIRECTURL_AFTER_REJECTION => $redirectUrlAfterRejection,
            PAYDIREKT_REDIRECTURL_AFTER_AGE_VERIFICATION_FAILURE => $redirectUrlAfterAgeVerificationFailure,
            PAYDIREKT_CALLBACKURL_UPDATES => $callbackUrlStatusUpdates,
            PAYDIREKT_PARAMETER_MERCHANT_RECONCILATION_REFERENCE_NUMBER => Checkout::getValidSepaString(
                substr($this->settings[PAYDIREKT_PARAMETER_MERCHANT_RECONCILATION_REFERENCE_NUMBER], 0, 30)
            ),
            PAYDIREKT_NOTE => Checkout::getValidSepaStringNote(substr($this->settings[PAYDIREKT_PARAMETER_ORDERNOTE], 0, 37)),
        );
    }

    /**
     * write given data into array
     *
     * @return array
     */
    private function buildCustomerShippingAddress()
    {
        $address = PaydirektUtils::splitStreetAndNumber(
            PAYDIREKT_COMPATIBILITY ? $this->order->shipping_address_1 : $this->order->get_shipping_address_1()
        );
        $data = array();
        if (!$this->order->needs_shipping_address()) {
            $address = PaydirektUtils::splitStreetAndNumber(
                PAYDIREKT_COMPATIBILITY ? $this->order->billing_address_1 : $this->order->get_billing_address_1()
            );
            $data = array(
                'addresseeGivenName' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_first_name : $this->order->get_billing_first_name(),
                'addresseeLastName' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_last_name : $this->order->get_billing_last_name(),
                'company' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_company : $this->order->get_billing_company(),
                'street' => $address['name'],
                'streetNr' => $address['number'],
                'additionalAddressInformation' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_address_2 : $this->order->get_billing_address_2(),
                'zip' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_postcode : $this->order->get_billing_postcode(),
                'city' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_city : $this->order->get_billing_city(),
                'countryCode' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_country : $this->order->get_billing_country(),
                'state' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_state : $this->order->get_billing_state(),
                'emailAddress' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_email : $this->order->get_billing_email()
            );
        } else {
            $data = array(
                'addresseeGivenName' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_first_name : $this->order->get_shipping_first_name(),
                'addresseeLastName' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_last_name : $this->order->get_shipping_last_name(),
                'company' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_company : $this->order->get_shipping_company(),
                'street' => $address['name'],
                'streetNr' => $address['number'],
                'additionalAddressInformation' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_address_2 : $this->order->get_shipping_address_2(),
                'zip' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_postcode : $this->order->get_shipping_postcode(),
                'city' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_city : $this->order->get_shipping_city(),
                'countryCode' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_country : $this->order->get_shipping_country(),
                'state' => PAYDIREKT_COMPATIBILITY ? $this->order->shipping_state : $this->order->get_shipping_state(),
                'emailAddress' => PAYDIREKT_COMPATIBILITY ? $this->order->billing_email : $this->order->get_billing_email()
            );
        }

        $this->logger->debug("Checkout: Shipping Data", $data);
        return $data;
    }
    /**
     * returns the basket type (PHYSICAL, DIGITAL or MIXED)
     *
     * @return string
     */
    private function getBasketType()
    {
        $virtual = false;
        $virtualCount = 0;
        $items = $this->order->get_items();
        $type = "PHYSICAL";

        foreach ($items as $item) {
            $product = $this->order->get_product_from_item($item);
            if ($product->is_virtual()) {
                $virtual = true;
                $virtualCount++;
            }
        }

        if (count($items) == $virtualCount) {
            $type = "DIGITAL";
        }
        if ($virtual === true && $virtualCount != count($items)) {
            $type = "MIXED";
        }

        return $type;
    }

    private function fillCheckoutBasket()
    {
        $items = $this->order->get_items();
        foreach ($items as $item) {
            $checkoutBasketItem = array(
                'quantity' => PAYDIREKT_COMPATIBILITY ? $item['qty'] : $item->get_quantity(),
                'name' => PAYDIREKT_COMPATIBILITY ? $item['name'] : $item->get_name(),
                'price' => number_format(
                    PAYDIREKT_COMPATIBILITY ?
                    $this->order->get_item_total($item, true, false) :
                    $item->get_total() + $item->get_total_tax(),
                    2, '.', ''
                ),
            );
            Checkout::basketItems($checkoutBasketItem);
        }
    }
}
