<?php
/*
 * Plugin Name: paydirekt
 * Plugin URI: https://paydirekt.logata.com
 * Description: paydirekt payment service
 * Version: 1.0.44
 * Author: Logata Digital Solutions GmbH
 * Author URI: https://www.logata.com
 * Text Domain: fulino-paydirekt
 * Domain Path: /languages
 * Copyright: Logata Digital Solutions GmbH
 * License URI: https://logata.com/paydirekt-lizenzbedingungen

 */

namespace Bitergo\PaydirektWooCommerce;

use Bitergo\PaydirektWooCommerce\classes\PaydirektConfigForm;
use Bitergo\PaydirektWooCommerce\classes\PaydirektUtils;
use Bitergo\PaydirektWooCommerce\Lib\PaydirektLogger;
use Bitergo\PaydirektWooCommerce\Lib\Utils;
use Bitergo\PaydirektWooCommerce\classes\PaydirektAdminMenu;
use Bitergo\PaydirektWooCommerce\Lib\Checkout;
use Bitergo\PaydirektWooCommerce\classes\ResponseStatusEnum;
use Bitergo\PaydirektWooCommerce\classes\components\PaydirektReportTable;
use Bitergo\PaydirektWooCommerce\classes\PaydirektCallback;
use Bitergo\PaydirektWooCommerce\classes\PaydirektCheckout;
use Bitergo\PaydirektWooCommerce\classes\PaydirektReturnAction;

if (!defined('ABSPATH')) {
    exit;
}

add_action('plugins_loaded', 'Bitergo\PaydirektWooCommerce\paydirekt_init', 0);
require_once(plugin_dir_path(__FILE__) . 'autoload.php');


function paydirekt_init()
{
    // If the WooCommerce payment gateway class is not available, do nothing
    if (!class_exists('\WC_Payment_Gateway')) {
        return;
    }

    /**
     * Localisation
     */
    load_plugin_textdomain(PAYDIREKT_PAYMENT_ID, false, dirname(plugin_basename(__FILE__)) . '/languages');

    /**
     * Gateway class
     */
    class WC_Gateway_Paydirekt extends \WC_Payment_Gateway
    {

        private $logger;

        /**
         * Constructor for the gateway.
         */
        public function __construct()
        {
            $this->countries;
            $this->max_amount;
            $this->id = PAYDIREKT_PAYMENT_ID;
            $this->icon = PAYDIREKT_PAYMENT_ICON_URL;
            $this->has_fields = false;
            $this->method_title = PAYDIREKT_PAYMENT_TITLE;
            $this->method_description = __(PAYDIREKT_PAYMENT_DESCRIPTION, 'fulino-paydirekt');
            $this->supports = array(
                'subscriptions',
                'products',
                'subscription_cancellation',
                'subscription_reactivation',
                'subscription_suspension',
                'subscription_amount_changes',
                'subscription_payment_method_change',
                'subscription_date_changes',
            );
            // Load the settings.
            $this->init_form_fields();
            $this->init_settings();
            // Define user set variables
            $this->title = PAYDIREKT_PAYMENT_TITLE;
            if (!empty($this->get_option(PAYDIREKT_PARAMETER_PAYMENT_TITLE))) {
                $this->title = $this->get_option(PAYDIREKT_PARAMETER_PAYMENT_TITLE);
            }
            $this->description = PAYDIREKT_PAYMENT_DESCRIPTION;
            if (!empty($this->get_option(PAYDIREKT_PARAMETER_PAYMENT_HTML))) {
                $this->description = $this->get_option(PAYDIREKT_PARAMETER_PAYMENT_HTML);
            }
            $this->apiKey = $this->get_option(PAYDIREKT_PARAMETER_APIKEY);
            $this->apiSecret = $this->get_option(PAYDIREKT_PARAMETER_APISECRET);
            $this->orderNote = $this->get_option(PAYDIREKT_PARAMETER_ORDERNOTE);
            $this->minimumAge = $this->get_option(PAYDIREKT_PARAMETER_MINIMUMAGE);
            $this->chooseCheckout = $this->get_option(PAYDIREKT_PARAMETER_CHOOSECHECKOUT);
            $this->logLevel = $this->get_option(PAYDIREKT_PARAMETER_LOGLEVEL);
            $this->notify_url = add_query_arg('wc-api', 'callback_' . $this->id, home_url('/'));
            $this->logger = PaydirektLogger::getInstance(get_class($this), $this->logLevel);

            // Actions
            /* 1.6.6 >= && <2.0.0 */
            add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            /* >= 2.0.0 */
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
            add_action('woocommerce_thankyou_' . $this->id, array($this, 'paydirekt_thankyou'));
            // Display an admin notice, if setup is required.
            add_action('admin_notices', array($this, 'display_admin_setup_notice'));
            add_action('woocommerce_api_callback', array($this, "paydirekt_notify"));
        }

        public function paydirekt_notify($api_request_url)
        {
            if (isset($_GET[PAYDIREKT_ACTION]) && sanitize_text_field($_GET[PAYDIREKT_ACTION]) == "paydirekt-notify") {
                $rawPostData = file_get_contents('php://input');
                $this->logger->debug($rawPostData);
                $callback = new PaydirektCallback($rawPostData, $this->settings);
                $callback->processNotify();
            }
            wp_die(null, null, 200);
        }

        /**
         * Create form fields for the payment gateway
         *
         * @return void
         */
        public function init_form_fields()
        {
            require_once plugin_dir_path(__FILE__) . "classes/PaydirektConfigForm.php";
            $this->form_fields = PaydirektConfigForm::getForm();
        }

        /**
         * Process the order payment status
         *
         * @param int $order_id
         * @return array
         */
        public function process_payment($order_id)
        {
            global $woocommerce;
            global $wp_rewrite;

            $order = new \WC_Order($order_id);
            $paydirektCheckout = new PaydirektCheckout($order);

            $checkout = $paydirektCheckout->createCheckout();
            if ($checkout == null) {
                \wc_add_notice(__(PAYDIREKT_PAYMENT_ERROR, 'woothemes') . $paydirektCheckout->getError()['description'], PAYDIREKT_ERROR);
                return;
            }

            $checkoutId = $checkout['checkoutId'];

            if (empty($checkoutId)) {
                \wc_add_notice(__(PAYDIREKT_PAYMENT_ERROR, 'woothemes') . "Checkout failed", PAYDIREKT_ERROR);
                return;
            }
            if (PAYDIREKT_COMPATIBILITY) {
                update_post_meta($order->id, '_transaction_id', $checkoutId);
            } else {
                $order->set_transaction_id($checkoutId);
                $order->save();
            }

            \WC()->session->set(PAYDIREKT_PAYMENT_ID . '_cart_contents', $woocommerce->cart->get_cart_contents());

            return array(
                PAYDIREKT_RESULT => PAYDIREKT_SUCCESS,
                PAYDIREKT_REDIRECT => $paydirektCheckout->getRedirectUrl(),
            );
        }

        public function validate_text_field($key, $value)
        {
            global $woocommerce;
            switch ($key) {
                case PAYDIREKT_PARAMETER_APIKEY:
                    if (Utils::isApiKey($value) === false) {
                        $this->errors[] = __('error_format_apikey', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_ORDERNOTE:
                    $croppedValue = PaydirektUtils::removeConfigurationVariablesFromString($value);
                    $sepa = Checkout::getValidSepaString($croppedValue);
                    if (strlen($value) > 37) {
                        $this->errors[] = __('error_format_ordernote', 'fulino-paydirekt');
                    }
                    if ($sepa != $croppedValue) {
                        $this->errors[] = __('error_format_ordernote_sepa', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_MINIMUMAGE:
                    if (!is_numeric($value) || !($value < 100 && $value > -1)) {
                        $this->errors[] = __('error_format_minimumage', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_MINIMUMAMOUNT:
                    $convertedValue = str_replace(",", ".", $value);
                    if (!is_numeric($convertedValue) || strpos($convertedValue, ".") === false) {
                        $this->errors[] = __('error_format_minimumamount', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_MAXIMUMAMOUNT:
                    $convertedValue = str_replace(",", ".", $value);
                    if (!is_numeric($convertedValue) || strpos($convertedValue, ".") === false) {
                        $this->errors[] = __('error_format_maximumamount', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_MERCHANT_RECONCILATION_REFERENCE_NUMBER:
                    $croppedValue = PaydirektUtils::removeConfigurationVariablesFromString($value);
                    $sepa = Checkout::getValidSepaString($croppedValue);
                    if (strlen($value) > 30) {
                        $this->errors[] = __('error_format_merchantreconciliationreferencenumber', 'fulino-paydirekt');
                    }
                    if ($sepa != $croppedValue) {
                        $this->errors[] = __('error_format_merchantreconciliationreferencenumber_sepa', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_RECONCILIATIONNUMBER:
                    $croppedValue = PaydirektUtils::removeConfigurationVariablesFromString($value);
                    $sepa = Checkout::getValidSepaString($croppedValue);
                    if (strlen($value) > 30) {
                        $this->errors[] = __('error_format_merchantreconciliationnumber', 'fulino-paydirekt');
                    }
                    if ($sepa != $croppedValue) {
                        $this->errors[] = __('error_format_merchantreconciliationnumber_sepa', 'fulino-paydirekt');
                    }
                    break;
                case PAYDIREKT_PARAMETER_CHOOSECHECKOUT:
                case PAYDIREKT_PARAMETER_LOGLEVEL:
                    break;
                default:
                    break;
            }

            $value = is_null($value) ? '' : $value;
            return wp_kses_post(trim(stripslashes($value)));
        }

        public function validate_password_field($key, $value)
        {
            if ($key == PAYDIREKT_PARAMETER_APISECRET && Utils::isApiSecret($value) === false) {
                $this->errors[] = __('error_format_apisecret', 'fulino-paydirekt');
                $this->display_errors();
            }

            $value = is_null($value) ? '' : $value;
            return trim(stripslashes($value));
        }

        public function process_admin_options()
        {
            $this->init_settings();

            $post_data = $this->get_post_data();

            foreach ($this->get_form_fields() as $key => $field) {
                if ('title' !== $this->get_field_type($field)) {
                    try {
                        $this->settings[$key] = $this->get_field_value($key, $field, $post_data);
                    } catch (\Exception $e) {
                        $this->add_error($e->getMessage());
                    }
                }
            }
            $this->display_errors();
            return update_option($this->get_option_key(), apply_filters('woocommerce_settings_api_sanitized_fields_' . $this->id, $this->settings));
        }

        /**
         * Process the refund for the order
         *
         * @param integer $order_id
         * @param float $amount
         * @param string $reason
         * @return void
         */
        public function process_refund($order_id, $amount = null, $reason = '')
        {
            return false;
        }

        /**
         * Output for the order received page.
         *
         * @return void
         */
        public function paydirekt_thankyou($order_id)
        {
            if ($description = $this->get_description()) {
                echo wp_kses(wpautop(wptexturize(wp_kses_post($description))), wp_kses_allowed_html('post'));
            }

            echo wp_kses('<h2>' . __('Our Details', PAYDIREKT_PAYMENT_ID) . '</h2>', wp_kses_allowed_html('post'));

            echo wp_kses('<ul class="order_details paydirekt_details">', wp_kses_allowed_html('post'));

            $fields = apply_filters('woocommerce_paydirekt_fields', array(
                'example_field' => __('Example field', PAYDIREKT_PAYMENT_ID),
            ));

            foreach ($fields as $key => $value) {
                if (!empty($this->$key)) {
                    echo wp_kses('<li class="' . esc_attr($key) . '">' . esc_attr($value) . ': <strong>' . wptexturize($this->$key) . '</strong></li>', wp_kses_allowed_html('post'));
                }
            }

            echo wp_kses('</ul>', array("</ul>"));
        }

        /**
         * Display an admin notice, if not on the integration screen and if the account isn't yet connected.
         * @access public
         * @return void
         */
        public function display_admin_setup_notice()
        {
            if (isset($_GET['page'])
                && 'wc-settings' == sanitize_text_field($_GET['page'])
                && isset($_GET[PAYDIREKT_SECTION])
                && PAYDIREKT_PAYMENT_ID == sanitize_text_field($_GET[PAYDIREKT_SECTION])) {
                return; // Don't show these notices on our admin screen.
            }

            // Find a different method of retrieving this value.
            $apiKey = $this->get_option(PAYDIREKT_PARAMETER_APIKEY);
            $apiSecret = $this->get_option(PAYDIREKT_PARAMETER_APISECRET);

            if ('' == $apiKey || '' == $apiSecret) {
                $url = $this->get_settings_url();
                echo wp_kses('<div class="updated fade"><p>' .
                sprintf(__('%spaydirekt is almost ready.%s To get started, %sconnect your paydirekt account%s.',
                    PAYDIREKT_PAYMENT_ID),
                    '<strong>',
                    '</strong>',
                    '<a href="' . esc_url($url) . '">',
                    '</a>') . '</p></div>' . "\n", wp_kses_allowed_html('post'));
            }
        } // End maybe_display_admin_notices()

        /**
         * Generate a URL to our specific settings screen.
         * @access public
         * @since  1.0.0
         * @return string Generated URL.
         */
        public function get_settings_url()
        {
            $url = admin_url('admin.php');
            $url = add_query_arg('page', 'wc-settings', $url);
            $url = add_query_arg('tab', 'checkout', $url);
            return add_query_arg(PAYDIREKT_SECTION, PAYDIREKT_PAYMENT_ID, $url);
        }
    } //end class

    /**
     * Add the Gateway to WooCommerce
     **/
    add_filter('woocommerce_payment_gateways', 'Bitergo\PaydirektWooCommerce\woocommerce_add_gateway_paydirekt');
    function woocommerce_add_gateway_paydirekt($methods)
    {
        $methods[] = 'Bitergo\PaydirektWooCommerce\WC_Gateway_Paydirekt';
        return $methods;
    }

    /*
     * Disable paydirekt payment method in checkout, if the amount or the currency do not match the config
     */
    add_filter('woocommerce_available_payment_gateways', 'Bitergo\PaydirektWooCommerce\woocommerce_filter_gateways', 1);
    function woocommerce_filter_gateways($gateways)
    {
        global $woocommerce;
        $minAmount = floatval(str_replace(",", ".", PaydirektUtils::getConfig(PAYDIREKT_PARAMETER_MINIMUMAMOUNT)));
        $maxAmount = floatval(str_replace(",", ".", PaydirektUtils::getConfig(PAYDIREKT_PARAMETER_MAXIMUMAMOUNT)));
        $cartTotal = $woocommerce->cart->cart_contents_total + $woocommerce->cart->tax_total;

        $currency = get_woocommerce_currency();
        if(isset($_GET["pay_for_order"])){
            $order = wc_get_order(get_query_var('order-pay'));
            $cartTotal = $order->total;
            $currency = $order->currency;
        }
        if ($cartTotal <= 0 || $cartTotal < $minAmount || $cartTotal > 50000 || $cartTotal > $maxAmount || $currency != "EUR") {
            unset($gateways['fulino-paydirekt']);
        }

        return $gateways;
    }

    /**
     * Add Button to Woocommerce Orderview
     */
    add_filter('woocommerce_admin_order_actions', 'Bitergo\PaydirektWooCommerce\add_custom_order_status_actions_button', 100, 2);
    function add_custom_order_status_actions_button($actions, $order)
    {
        if (PAYDIREKT_COMPATIBILITY) {
            $payment_method = $order->payment_method;
        } else {
            $payment_method = $order->get_payment_method();
        }

        if ($payment_method == PAYDIREKT_PAYMENT_ID) {
            $order_id = PAYDIREKT_COMPATIBILITY ? $order->id : $order->get_id();
            $actions['paydirekt_overview'] = array(
                'url' => wp_nonce_url(admin_url('admin.php?page=paydirekt-orderview&order_id=' . $order_id), 'woocommerce-mark-order-status'),
                'name' => __('paydirekt Übersicht', PAYDIREKT_WOOCOMMERCE),
                PAYDIREKT_ACTION => "view paydirekt", // keep "view" class for a clean button CSS
            );
            return $actions;
        }
        return $actions;
    }

    // Set Here the WooCommerce icon for your action button
    add_action('admin_init', 'Bitergo\PaydirektWooCommerce\paydirekt_add_admin_css');
    function paydirekt_add_admin_css()
    {
        wp_enqueue_style('paydirekt-admin', plugins_url('resources/css/paydirekt_admin.css', __FILE__));
    }

    add_action('init', 'Bitergo\PaydirektWooCommerce\check_for_woopaydirekt');
    function check_for_woopaydirekt()
    {
        if (isset($_GET['woopaydirekt'])) {
            // Start the gateways
            \WC()->payment_gateways();

            $paydirektReturnAction = new PaydirektReturnAction();
            $returnVal = $paydirektReturnAction->check_response();
            if ($returnVal != null && isset($returnVal[PAYDIREKT_RESULT])) {
                switch ($returnVal[PAYDIREKT_RESULT]) {
                    case ResponseStatusEnum::REJECT:
                        \wc_add_notice(
                            __(PAYDIREKT_PAYMENT_ERROR, PAYDIREKT_PAYMENT_ID) . __("payment was rejected from paydirekt system", PAYDIREKT_PAYMENT_ID),
                            PAYDIREKT_ERROR
                        );
                        break;
                    case ResponseStatusEnum::CANCEL:
                        \wc_add_notice(
                            __(PAYDIREKT_PAYMENT_ERROR, PAYDIREKT_PAYMENT_ID) . __("payment was canceled", PAYDIREKT_PAYMENT_ID),
                            PAYDIREKT_ERROR
                        );

                        break;
                    case ResponseStatusEnum::AGE:
                        \wc_add_notice(
                            __(PAYDIREKT_PAYMENT_ERROR, PAYDIREKT_PAYMENT_ID) . __("payment was rejected by age", PAYDIREKT_PAYMENT_ID),
                            PAYDIREKT_ERROR
                        );
                        break;
                    default:
                        break;
                }

                wp_redirect($returnVal[PAYDIREKT_REDIRECT]);
                die();
            }
        }
    }

    add_action('init', 'Bitergo\PaydirektWooCommerce\checkCsvDownload');
    function checkCsvDownload()
    {
        if (isset($_GET["page"]) && sanitize_text_field($_GET["page"]) == "paydirekt-report" && sanitize_text_field($_GET[PAYDIREKT_ACTION]) == "csv") {
            $paydirektReportTable = new PaydirektReportTable();
            $paydirektReportTable->getPaydirektReport("csv");
        }
    }

    add_action('admin_menu', 'Bitergo\PaydirektWooCommerce\add_admin_menu');
    function add_admin_menu()
    {
        $adminMenu = new PaydirektAdminMenu();
    }

    add_action('wp_ajax_paydirekt_detail_view', array('Bitergo\PaydirektWooCommerce\classes\PaydirektOrderView', PAYDIREKT_GET_AJAX_DATA));
    add_action('wp_ajax_paydirekt_report_view', array('Bitergo\PaydirektWooCommerce\classes\PaydirektReport', PAYDIREKT_GET_AJAX_DATA));
    add_action('wp_ajax_paydirekt_check_config', array('Bitergo\PaydirektWooCommerce\classes\PaydirektConfigForm', PAYDIREKT_GET_AJAX_DATA));

    add_action('woocommerce_check_cart_items', 'Bitergo\PaydirektWooCommerce\recover_cart');
    function recover_cart()
    {
        if (isset($_GET["rc"]) && !is_admin() && $_GET['rc']) {
            $session_cart = \WC()->session->get(PAYDIREKT_PAYMENT_ID . '_cart_contents');
            if (!empty($session_cart)) {
                \WC()->cart->set_cart_contents($session_cart);
                \WC()->session->set(PAYDIREKT_PAYMENT_ID . '_cart_contents', "");
            }
        }
    }

}
